//
//  ImageViewController.swift
//  Images
//
//  Created by Bart Jacobs on 26/01/2021.
//

import UIKit

internal final class ImageViewController: UIViewController {

    // MARK: - Properties
    
    var image: Image?
    
    // MARK: -
    
    @IBOutlet private var imageView: UIImageView! {
        didSet {
            // Configure Image View
            imageView.contentMode = .scaleAspectFit
        }
    }
    
    @IBOutlet private var activityIndicatorView: UIActivityIndicatorView! {
        didSet {
            // Configure Activity Indicator View
            activityIndicatorView.startAnimating()
            activityIndicatorView.hidesWhenStopped = true
        }
    }

    // MARK: -
    
    private var dataTask: URLSessionDataTask?

    // MARK: - Deinitialization

    deinit {
        dataTask?.cancel()
    }
    
    // MARK: - View Life Cycle
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if let url = image?.fullSizeUrl {
            // Fetch Image
            dataTask = fetchImage(with: url)
        }
    }

    // MARK: - Helper Methods
    
    private func fetchImage(with url: URL) -> URLSessionDataTask {
        // Create URL Request
        let request = URLRequest(url: url, cachePolicy: .returnCacheDataElseLoad, timeoutInterval: 30.0)
        
        // Create Data Task
        let dataTask = URLSession.shared.dataTask(with: request) { [weak self] data, _, error in
            if let data = data {
                // Create Image from Data
                let image = UIImage(data: data)
                
                DispatchQueue.main.async {
                    // Update Image View
                    self?.imageView.image = image
                    
                    // Hide Activity Indicator View
                    self?.activityIndicatorView.stopAnimating()
                }
            } else if let error = error {
                print("Unable to Fetch Image \(error)")
            }
        }
        
        dataTask.resume()
        
        return dataTask
    }

}
