//
//  ImageTableViewCell.swift
//  Images
//
//  Created by Bart Jacobs on 25/01/2021.
//

import UIKit

final class ImageTableViewCell: UITableViewCell {

    // MARK: - Static Properties
    
    static var reuseIdentifier: String {
        return String(describing: self)
    }
    
    // MARK: - Properties
    
    @IBOutlet private var titleLabel: UILabel!
    
    // MARK: -
    
    @IBOutlet private var thumbnailImageView: UIImageView!
    
    // MARK: -
    
    @IBOutlet var activityIndicatorView: UIActivityIndicatorView!

    // MARK: -
    
    private var dataTask: URLSessionDataTask?
    
    // MARK: - Public API
    
    func configure(title: String, url: URL) {
        // Configure Title Label
        titleLabel.text = title
        
        // Animate Activity Indicator View
        activityIndicatorView.startAnimating()
        
        // Fetch Image
        dataTask = fetchImage(with: url)
    }
    
    // MARK: - Overrides
    
    override func prepareForReuse() {
        super.prepareForReuse()
        
        // Cancel Data Task
        dataTask?.cancel()
        
        // Reset Thumnail Image View
        thumbnailImageView.image = nil
    }
    
    // MARK: - Helper Methods

    private func fetchImage(with url: URL) -> URLSessionDataTask {
        // Create URL Request
        let request = URLRequest(url: url, cachePolicy: .returnCacheDataElseLoad, timeoutInterval: 30.0)
        
        // Create Data Task
        let dataTask = URLSession.shared.dataTask(with: request) { [weak self] data, _, error in
            if let data = data {
                // Create Image from Data
                let image = UIImage(data: data)
                
                DispatchQueue.main.async {
                    // Update Thumbnail
                    self?.thumbnailImageView.image = image
                    
                    // Hide Activity Indicator View
                    self?.activityIndicatorView.stopAnimating()
                }
            } else if let error = error {
                print("Unable to Fetch Image \(error)")
            }
        }
        
        dataTask.resume()
        
        return dataTask
    }
    
}
